import KeyMap from '../../../utils/key-map';
import ConstraintMap from '../../../utils/constraint-map';
import { IDatabaseAdapter, DatabaseConfig } from '../../../types/database';
import Relation from '../../../features/orm/relation';
import { ClassId } from '../../../types/class';
export default class MySQLDatabaseAdapter implements IDatabaseAdapter {
    /**
     * Private properties
     */
    private client;
    private constraints;
    /**
     * Constructor
     * @param {Object} config
     */
    constructor(config: DatabaseConfig);
    /**
     * Get current timesamp
     * @returns {String}
     */
    readonly currentTimestamp: string;
    initialize(): Promise<void>;
    private escapeKey;
    private regularEscape;
    private collectionEscape;
    private subqueryEscape;
    private parseConstraint;
    /**
     * Generate Find Statement
     * @description Decoupled from the `find` method
     * in order to allow subquery select statements
     * @param {String} source
     * @param {String} className
     * @param {Array} select
     * @param {Array} relations
     * @param {KeyMap} where
     * @param {boolean} isSubquery
     */
    private getFindClause;
    /**
     * Generate Sorting
     * @param {String} className
     * @param {Array} sort
     */
    private getSortClause;
    /**
     * Map row keys into appropriate relations
     * @param {Object} row
     */
    private mapRows;
    find(source: [string, string], columns: Map<string, string>, relations: Map<string, Relation>, constraints: ConstraintMap, sorting: string[], skipped: number, limitation: number): Promise<KeyMap[]>;
    create(source: string, keys: KeyMap): Promise<ClassId>;
    update(source: string, keys: KeyMap, id: ClassId): Promise<void>;
    destroy(source: string, keys: KeyMap, id: ClassId): Promise<void>;
}
