"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var error_1 = __importDefault(require("../../../utils/error"));
var ConsoleLoggerAdapter = /** @class */ (function () {
    /**
     * Constructor
     */
    function ConsoleLoggerAdapter(appName) {
        this.appLogName = appName;
    }
    Object.defineProperty(ConsoleLoggerAdapter, "Levels", {
        get: function () {
            return Object.freeze({
                Verbose: 'verbose',
                Warning: 'warning',
                Error: 'error',
                Disabled: 'disabled',
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConsoleLoggerAdapter.prototype, "statics", {
        get: function () {
            return this.constructor;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConsoleLoggerAdapter.prototype, "appName", {
        get: function () {
            return this.appLogName;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConsoleLoggerAdapter.prototype, "timestamp", {
        get: function () {
            var date = new Date();
            return date.toISOString().slice(0, 19).replace('T', ' ');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConsoleLoggerAdapter.prototype, "level", {
        get: function () {
            if (!this.logLevel)
                return this.statics.Levels.Verbose;
            return this.logLevel;
        },
        set: function (value) {
            if (!Object.values(this.statics.Levels).includes(value))
                return;
            this.logLevel = value;
        },
        enumerable: true,
        configurable: true
    });
    ConsoleLoggerAdapter.prototype.header = function (type) {
        return "[" + this.appName + " " + this.timestamp + "][" + type + "]";
    };
    ConsoleLoggerAdapter.prototype.bare = function () {
        var message = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            message[_i] = arguments[_i];
        }
        /* eslint-disable no-console */
        console.log.apply(this, __spread(message));
        /* eslint-enable no-console */
    };
    ConsoleLoggerAdapter.prototype.info = function () {
        var message = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            message[_i] = arguments[_i];
        }
        /* eslint-disable no-console */
        if (this.level === this.statics.Levels.Verbose)
            console.log.apply(this, __spread([this.header('INFO')], message));
        /* eslint-enable no-console */
    };
    ConsoleLoggerAdapter.prototype.warn = function () {
        var message = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            message[_i] = arguments[_i];
        }
        /* eslint-disable no-console */
        if (this.level !== this.statics.Levels.Error && this.level !== this.statics.Levels.Disabled)
            console.warn.apply(this, __spread([this.header('WARNING')], message));
        /* eslint-enable no-console */
    };
    ConsoleLoggerAdapter.prototype.error = function (err) {
        var message = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            message[_i - 1] = arguments[_i];
        }
        // Prepare header
        var header = this.header('ERROR');
        // Get code if applicable
        if (err instanceof error_1.default) {
            header = header.concat("[Code: " + err.code + "]");
        }
        /* eslint-disable no-console */
        if (this.level === this.statics.Levels.Disabled)
            return;
        else if (this.level === this.statics.Levels.Verbose)
            console.error.apply(this, __spread([header], message, [err.stack]));
        else
            console.error.apply(this, __spread([header], message));
        /* eslint-enable no-console */
    };
    return ConsoleLoggerAdapter;
}());
exports.default = ConsoleLoggerAdapter;
//# sourceMappingURL=index.js.map