import Class from './class';
import Query from './query';
import User from '../auth/user';
import { IDatabaseAdapter } from '../../types/database';
import Collection from '../../utils/collection';
import { ClassMapType, ClassOptions, ClassId } from '../../types/class';
export default class ClassManager {
    private database;
    private classes;
    constructor(database: IDatabaseAdapter);
    /**
     * Get key map from instance
     * @param classInstance
     */
    private getKeyMapFrom;
    /**
     * Connect to the database
     */
    initialize(): Promise<void>;
    /**
     * Register classes to data mapper
     * @param classMap
     */
    register(classMap: ClassMapType<any>): void;
    /**
     * Get class from registry
     * @param className
     */
    get<C extends typeof Class>(className: string): C;
    /**
     * Find objects
     * @param query
     * @param opts
     */
    find<C extends typeof Class, U extends User>(query: Query<C>, opts?: ClassOptions<U>): Promise<Collection<C['prototype']>>;
    /**
     * Find the first object that matches
     * @param query
     * @param opts
     */
    first<C extends typeof Class, U extends User>(query: Query<C>, opts?: ClassOptions<U>): Promise<C['prototype'] | null>;
    /**
     * Find a single object
     * @param classType
     * @param {number|string} id
     */
    getById<C extends typeof Class, U extends User>(classType: C, id: ClassId, include?: string[], select?: string[], opts?: ClassOptions<U>): Promise<C['prototype'] | null>;
    /**
     * Save the object
     * @param classInstance
     */
    save<C extends Class, U extends User | undefined>(classInstance: C, opts?: ClassOptions<U>): Promise<C>;
    /**
     * Destroy the object
     * @param classInstance
     */
    destroy<C extends Class, U extends User | undefined>(classInstance: C, opts?: ClassOptions<U>): Promise<void>;
    /**
     * Generates a batch loader function
     * Compatible with the `dataloader` library of Facebook
     * @param query
     */
    batch<C extends typeof Class, U extends User>(query: Query<C>, opts?: ClassOptions<U>): (ids: number[]) => Promise<(C["prototype"] | null)[]>;
}
