"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var format_1 = require("../../../utils/format");
var class_1 = __importStar(require("../class"));
var string_1 = __importDefault(require("./types/string"));
var date_1 = __importDefault(require("./types/date"));
var number_1 = __importDefault(require("./types/number"));
var json_1 = __importDefault(require("./types/json"));
var boolean_1 = __importDefault(require("./types/boolean"));
var relation_1 = __importDefault(require("./types/relation"));
var relation_2 = __importStar(require("../relation"));
var constants_1 = require("../../../utils/constants");
/**
 * Key Manager
 * @description Handler for getters and setters
 */
var KeyManager = /** @class */ (function () {
    function KeyManager(name, type) {
        this.isNewFlag = false;
        this.setterDefinition = function (value) { return value; };
        this.getterDefinition = function (value) { return value; };
        this.keyName = name;
        this.keyType = type || 'any';
    }
    Object.defineProperty(KeyManager.prototype, "type", {
        get: function () {
            return this.keyType;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyManager.prototype, "isNew", {
        get: function () {
            return this.isNewFlag;
        },
        set: function (value) {
            this.isNewFlag = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyManager.prototype, "name", {
        get: function () {
            return this.keyName;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyManager.prototype, "setter", {
        get: function () {
            return this.setterDefinition;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyManager.prototype, "getter", {
        get: function () {
            return this.getterDefinition;
        },
        enumerable: true,
        configurable: true
    });
    return KeyManager;
}());
exports.KeyManager = KeyManager;
/**
 * Key Decorator
 * @param opts
 */
exports.keyDecorator = function (opts) {
    if (opts === void 0) { opts = {}; }
    return function (classInstance, name) {
        // Get type name
        var type = opts.type;
        var from = opts.from, to = opts.to;
        // Convert key name to snake case, prepare key manager
        var keyName = format_1.toSnakeCase(name);
        var sourceName = from || keyName;
        var keyManager = new KeyManager(keyName);
        // Infer data type
        var inferredType = Reflect.getMetadata('design:type', classInstance, name);
        // Get type from metadata
        if (typeof inferredType !== 'undefined')
            type = inferredType.name.toLowerCase();
        // Get definition
        var definition = class_1.ClassDefinitionManager.get(classInstance.statics());
        if (!definition.keys.includes(keyName) && !definition.timestamps.includes(keyName)) {
            definition.keys.push(keyName);
        }
        // Determine key manager
        if (type === 'string')
            keyManager = string_1.default(sourceName);
        else if (type === 'date')
            keyManager = date_1.default(sourceName);
        else if (type === 'boolean')
            keyManager = boolean_1.default(sourceName);
        else if (type === 'number')
            keyManager = number_1.default(sourceName);
        else if (type === 'array')
            keyManager = json_1.default(sourceName);
        else if (type === 'json')
            keyManager = json_1.default(sourceName);
        else if (inferredType && (new inferredType) instanceof class_1.default) {
            // Set default values
            from = from || relation_2.default.formatKey(relation_2.RelationDefinition.OwnerSymbol, relation_2.default.formatAsId(keyName));
            to = to || relation_2.default.formatKey(keyName, constants_1.InternalId);
            // Extract keys
            var _a = __read(relation_2.default.parseKey(from), 2), sourceClassName = _a[0], sourceKey = _a[1];
            var _b = __read(relation_2.default.parseKey(to), 2), parentClassName = _b[0], parentKey = _b[1];
            // Prepare relation definition
            var relationDefinition = new relation_2.RelationDefinition(function () { return inferredType; }, sourceClassName, sourceKey, parentClassName, parentKey);
            // Prepare key manager
            keyManager = relation_1.default(keyName, relationDefinition);
            // Set relation definition
            definition.relations[keyName] = relationDefinition;
        }
        // Set definition
        class_1.ClassDefinitionManager.set(classInstance.statics(), definition);
        // Get existing descriptor
        var descriptor = Object.getOwnPropertyDescriptor(classInstance, name);
        // Extend getter and setter
        Object.defineProperty(classInstance, name, {
            set: function (value) {
                // Parse value
                value = keyManager.setter(value);
                // If descriptor is defined
                if (descriptor && typeof descriptor.set === 'function') {
                    // Set value
                    descriptor.set.apply(this, [value]);
                }
                else
                    this.keys.set(keyManager.name, value);
            },
            get: function () {
                // Prepare value
                var value = this.keys.get(keyManager.name);
                // If descriptor is defined
                if (descriptor && typeof descriptor.get === 'function') {
                    // Get formatted value
                    value = descriptor.get.apply(this);
                }
                // Format value
                value = keyManager.getter(value);
                // Return value
                return value;
            },
            enumerable: true,
            configurable: true,
        });
    };
};
function key() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    // As property decorator
    if (args.length === 2 || args.length === 3) {
        return exports.keyDecorator()(args[0], args[1]);
    }
    else if (args.length === 1) {
        return exports.keyDecorator(args[0]);
    }
    else {
        return exports.keyDecorator();
    }
}
exports.key = key;
//# sourceMappingURL=key.js.map