"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var class_1 = require("../../class");
var format_1 = require("../../../../utils/format");
/**
 * Validates the if the value of a number is between the given values
 * @param classInstance
 * @param name
 */
exports.between = function (min, max) { return function (classInstance, name) {
    // Convert key name to snake case, then add to the key map
    var keyName = format_1.toSnakeCase(name);
    // Infer data type
    var inferredType = Reflect.getMetadata('design:type', classInstance, name);
    // Get type from metadata
    if (!inferredType || inferredType.name.toLowerCase() !== 'number')
        throw new Error("Property `" + name + "` cannot be modified by `@range()` because it is not a number type");
    // Get existing descriptor
    var descriptor = Object.getOwnPropertyDescriptor(classInstance, name);
    // Get definition
    var definition = class_1.ClassDefinitionManager.get(classInstance.statics());
    if (!definition.keys.includes(keyName))
        throw new Error("Property `" + name + "` cannot be modified by `@range()` because it is not decorated with `@key`");
    // Override getter and setter
    Object.defineProperty(classInstance, name, {
        set: function (value) {
            // Validate value
            if (typeof value === 'number') {
                if (value < min || value > max)
                    throw new Error("Key `" + keyName + "` must be between `" + min + "` and `" + max + "`");
            }
            // Set value
            descriptor && descriptor.set && descriptor.set.apply(this, [value]);
        },
        get: function () {
            // Get value
            var value = descriptor && descriptor.get && descriptor.get.apply(this) || undefined;
            // Return value
            return value;
        },
        enumerable: true,
        configurable: true,
    });
}; };
//# sourceMappingURL=between.js.map