"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var class_1 = require("../../class");
var format_1 = require("../../../../utils/format");
var error_1 = __importDefault(require("../../../../utils/error"));
/**
 * Adds the property to the list of computed keys
 * @param classInstance
 * @param name
 */
exports.computed = function (classInstance, name) {
    // Get key name
    var keyName = format_1.toSnakeCase(name);
    // Set definition
    var definition = class_1.ClassDefinitionManager.get(classInstance.statics());
    if (!definition.computed.includes(keyName))
        definition.computed.push(keyName);
    class_1.ClassDefinitionManager.set(classInstance.statics(), definition);
    // Get existing descriptor
    var descriptor = Object.getOwnPropertyDescriptor(classInstance, name);
    // Extend getter and setter
    Object.defineProperty(classInstance, name, {
        set: function (value) {
            // Prevent setting data
            throw new error_1.default(error_1.default.Code.InvalidObjectKey, "Key `" + keyName + "` cannot be set because it is a `computed` key");
        },
        get: function () {
            // If descriptor is defined
            if (descriptor && typeof descriptor.get === 'function') {
                // Get formatted value
                return descriptor.get.apply(this);
            }
            else
                throw new error_1.default(error_1.default.Code.MissingConfiguration, "Key `" + keyName + "` is a `computed` key but does not have a defined getter");
        },
        enumerable: true,
        configurable: true,
    });
};
//# sourceMappingURL=computed.js.map