"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var class_1 = require("../../class");
var format_1 = require("../../../../utils/format");
/**
 * Validates the length of a string
 * @param classInstance
 * @param name
 */
exports.rounded = function (precision, rule) {
    if (rule === void 0) { rule = 'off'; }
    return function (classInstance, name) {
        // Convert key name to snake case, then add to the key map
        var keyName = format_1.toSnakeCase(name);
        // Infer data type
        var inferredType = Reflect.getMetadata('design:type', classInstance, name);
        // Get type from metadata
        if (!inferredType || inferredType.name.toLowerCase() !== 'number')
            throw new Error("Property `" + name + "` cannot be modified by `@round()` because it is not a number type");
        // Get existing descriptor
        var descriptor = Object.getOwnPropertyDescriptor(classInstance, name);
        // Get definition
        var definition = class_1.ClassDefinitionManager.get(classInstance.statics());
        if (!definition.keys.includes(keyName))
            throw new Error("Property `" + name + "` cannot be modified by `@round()` because it is not decorated with `@key`");
        // Override getter and setter
        Object.defineProperty(classInstance, name, {
            set: function (value) {
                // Validate value
                if (typeof value === 'number') {
                    if (rule === 'off') {
                        value = Number(Number(value).toFixed(precision));
                    }
                    else if (rule === 'up') {
                        var multiplier = 10 * precision;
                        value = Math.ceil(Number(value) * multiplier) / multiplier;
                    }
                    else {
                        var multiplier = 10 * precision;
                        value = Math.floor(Number(value) * multiplier) / multiplier;
                    }
                }
                // Set value
                descriptor && descriptor.set && descriptor.set.apply(this, [value]);
            },
            get: function () {
                // Get value
                var value = descriptor && descriptor.get && descriptor.get.apply(this);
                // Return value
                return value;
            },
            enumerable: true,
            configurable: true,
        });
    };
};
//# sourceMappingURL=rounded.js.map