"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var key_1 = require("../key");
var error_1 = __importDefault(require("../../../../utils/error"));
var specials_1 = require("../../specials");
function NumberKey(name, opts) {
    if (opts === void 0) { opts = {}; }
    var min = opts.min, max = opts.max;
    var key = new key_1.KeyManager(name, 'number');
    key.setterDefinition = function (value) {
        // If null, set value to null
        if (typeof value === 'undefined' || value === null)
            return null;
        else if (typeof value === 'object') {
            if (!specials_1.Increment.isImplementedBy(value))
                throw new error_1.default(error_1.default.Code.InvalidObjectKey, "Key `" + name + "` is not a valid increment object");
            else
                return new specials_1.Increment(name, value, min, max);
        }
        else if (isNaN(value)) {
            // If the number is not valid, throw an error
            throw new error_1.default(error_1.default.Code.InvalidObjectKey, "Key `" + name + "` is not a valid number");
        }
        else if (typeof min !== 'undefined' && value < min) {
            throw new error_1.default(error_1.default.Code.InvalidObjectKey, "Key `" + name + "` must be greater than or equal to " + min);
        }
        else if (typeof max !== 'undefined' && value > max) {
            throw new error_1.default(error_1.default.Code.InvalidObjectKey, "Key `" + name + "` must be less than or equal to " + max);
        }
        else
            return Number(value);
    };
    key.getterDefinition = function (value) {
        if (typeof value === 'undefined' || value === null)
            return value;
        else
            return Number(value);
    };
    return key;
}
exports.default = NumberKey;
//# sourceMappingURL=number.js.map