"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var class_1 = require("./class");
var constants_1 = require("../../utils/constants");
var format_1 = require("../../utils/format");
var error_1 = __importDefault(require("../../utils/error"));
var pointer_1 = __importDefault(require("./keys/types/pointer"));
var Pointer = /** @class */ (function () {
    /**
     * Constructor
     * @param {Class} classType
     * @param {String} aliasKey
     */
    function Pointer(classType, sourceClassName, sourceKey, parentClassName, parentKey, secondary) {
        if (secondary === void 0) { secondary = false; }
        this.classType = classType;
        this.sourceClassName = sourceClassName;
        this.sourceKey = sourceKey;
        this.parentClassName = parentClassName;
        this.parentKey = parentKey;
        this.secondaryFlag = secondary;
    }
    Pointer.isUsedBy = function (key) {
        return key.indexOf(constants_1.PointerDelimiter) > 0;
    };
    Pointer.isValid = function (key) {
        return this.isUsedBy(key) && key.split(constants_1.PointerDelimiter).length === 2;
    };
    Pointer.parseKey = function (keyName) {
        // Get key parts
        var _a = __read(keyName.split(constants_1.PointerDelimiter, 2), 2), className = _a[0], key = _a[1];
        return [className, key];
    };
    Pointer.formatKey = function (className, key) {
        return "" + className + constants_1.PointerDelimiter + key;
    };
    Pointer.formatAsId = function (key) {
        return key + "_" + constants_1.InternalKeys.Id;
    };
    Pointer.prototype.statics = function () {
        return this.constructor;
    };
    Object.defineProperty(Pointer.prototype, "class", {
        get: function () {
            return this.classType;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pointer.prototype, "secondary", {
        get: function () {
            return this.secondaryFlag;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * The foreign key inside the current class
     */
    Pointer.prototype.sourceClassKey = function (className) {
        var sourceClass = this.sourceClassName !== PointerDefinition.OwnerSymbol ? this.sourceClassName : className;
        return this.statics().formatKey(sourceClass, this.sourceKey);
    };
    /**
     * The key inside the other class that the foreign key matches
     */
    Pointer.prototype.parentClassKey = function () {
        return this.statics().formatKey(this.parentClassName, this.parentKey);
    };
    Pointer.prototype.isImplementedBy = function (value) {
        if (value === null)
            return true;
        if (typeof value !== 'object')
            return false;
        if (value[constants_1.InternalKeys.Pointers.Type] !== 'Pointer')
            return false;
        if (value[constants_1.InternalKeys.Pointers.ClassName] !== this.class.className)
            return false;
        if (value[constants_1.InternalKeys.Id] === null || typeof value[constants_1.InternalKeys.Id] === 'undefined')
            return false;
        return true;
    };
    return Pointer;
}());
exports.default = Pointer;
var PointerDefinition = /** @class */ (function () {
    function PointerDefinition(classDefinition, sourceClassName, sourceKey, parentClassName, parentKey) {
        this.classCaller = classDefinition;
        this.sourceClassName = sourceClassName;
        this.sourceKey = sourceKey;
        this.parentClassName = parentClassName;
        this.parentKey = parentKey;
    }
    PointerDefinition.prototype.toPointer = function () {
        var classType = this.classCaller();
        var definition = class_1.ClassDefinitionManager.get(classType);
        var _a = this, sourceClassName = _a.sourceClassName, sourceKey = _a.sourceKey, parentClassName = _a.parentClassName, parentKey = _a.parentKey;
        var actualSourceKey = sourceKey;
        var secondary = false;
        // Check if source is the owner
        if (sourceClassName !== PointerDefinition.OwnerSymbol)
            secondary = true;
        // Check if pointer is secondary
        if (secondary) {
            if (typeof definition.relations[sourceKey] !== 'undefined') {
                // Get parent definition
                var parentDefinition = definition.relations[sourceKey];
                actualSourceKey = parentDefinition.toPointer().sourceKey;
            }
            else
                throw new error_1.default(error_1.default.Code.ForbiddenOperation, "Pointer `" + this.sourceKey + "` does not exist in " + sourceClassName);
        }
        // Create a pointer
        var pointer = new Pointer(classType, sourceClassName, actualSourceKey, parentClassName, parentKey, secondary);
        // Return pointer
        return pointer;
    };
    PointerDefinition.OwnerSymbol = '*';
    return PointerDefinition;
}());
exports.PointerDefinition = PointerDefinition;
/**
 * belongsTo decorator for pointers
 * @param classCaller
 */
exports.belongsTo = function (classCaller, from, to) {
    return function (classInstance, name) {
        // Get pointer name
        var keyName = format_1.toSnakeCase(name);
        // Set default values
        from = from || Pointer.formatKey(PointerDefinition.OwnerSymbol, Pointer.formatAsId(keyName));
        to = to || Pointer.formatKey(keyName, constants_1.InternalKeys.Id);
        // Extract keys
        var _a = __read(Pointer.parseKey(from), 2), sourceClassName = _a[0], sourceKey = _a[1];
        var _b = __read(Pointer.parseKey(to), 2), parentClassName = _b[0], parentKey = _b[1];
        // Prepare pointer definition
        var pointerDefinition = new PointerDefinition(classCaller, sourceClassName, sourceKey, parentClassName, parentKey);
        // Prepare key manager
        var keyManager = pointer_1.default(keyName, pointerDefinition);
        // Set definition
        var definition = class_1.ClassDefinitionManager.get(classInstance.statics());
        if (!definition.keys.includes(keyName))
            definition.keys.push(keyName);
        definition.relations[keyName] = pointerDefinition;
        class_1.ClassDefinitionManager.set(classInstance.statics(), definition);
        // Override pointer getter and setter
        return {
            set: function (value) {
                value = keyManager.setter(value);
                this.keys.set(keyManager.name, value);
            },
            get: function () {
                return keyManager.getter(this.keys.get(keyManager.name));
            },
            enumerable: true,
            configurable: true,
        };
    };
};
//# sourceMappingURL=pointer.js.map