"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var relation_1 = __importDefault(require("./relation"));
var constraint_map_1 = __importStar(require("../../utils/constraint-map"));
var constants_1 = require("../../utils/constants");
var compound_key_1 = __importDefault(require("../../utils/compound-key"));
/**
 * Get columns
 */
exports.getColumnsFrom = function (className, keys, relationsMap) {
    var e_1, _a;
    // Prepare columns
    var columns = new Map([]);
    try {
        // Iterate through select keys
        for (var keys_1 = __values(keys), keys_1_1 = keys_1.next(); !keys_1_1.done; keys_1_1 = keys_1.next()) {
            var key = keys_1_1.value;
            // Prepare source key
            var sourceKey = key;
            // Check if key is from a relation
            if (relation_1.default.isUsedBy(key)) {
                // Get source class and key
                var _b = __read(relation_1.default.parseKey(key), 2), sourceClassName = _b[0], sourceKeyName = _b[1];
                // If key is id, use the source class key
                if (sourceKeyName === constants_1.InternalId) {
                    var relationDefinition = relationsMap[sourceClassName];
                    var relation = relationDefinition.toRelation();
                    // Set the source key
                    sourceKey = relation.sourceClassKey(className);
                }
            }
            else {
                // Check if the key is a relation
                var relationDefinition = relationsMap[key];
                if (typeof relationDefinition !== 'undefined') {
                    // Get relation
                    var relation = relationDefinition.toRelation();
                    // Set the source key
                    sourceKey = relation.sourceClassKey(className);
                }
                else
                    sourceKey = relation_1.default.formatKey(className, key);
            }
            // Set column values
            columns.set(sourceKey, key);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (keys_1_1 && !keys_1_1.done && (_a = keys_1.return)) _a.call(keys_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return columns;
};
/**
 * Get relations
 * @param keys
 */
exports.getRelationsFrom = function (keys, relationsMap) {
    var e_2, _a;
    // Prepare relations
    var relations = new Map([]);
    try {
        // Iterate through the selected keys
        for (var keys_2 = __values(keys), keys_2_1 = keys_2.next(); !keys_2_1.done; keys_2_1 = keys_2.next()) {
            var key = keys_2_1.value;
            // Check if the key is a key from a relation
            if (relation_1.default.isUsedBy(key)) {
                // Get alias
                var _b = __read(relation_1.default.parseKey(key), 2), sourceClassName = _b[0], sourceKeyName = _b[1];
                // Get relation
                var keyRelationDefinition = relationsMap[sourceClassName];
                var relation = keyRelationDefinition.toRelation();
                // If the key provided is an `id`, skip it to optimize the query
                // (i.e. no need to use left join for `id` keys)
                if (sourceKeyName === constants_1.InternalId) {
                    continue;
                }
                else if (relation.secondary) {
                    // Get parent relation
                    var parentDefinition = relationsMap[relation.sourceClassName];
                    var parentRelation = parentDefinition.toRelation();
                    relations.set(relation.sourceClassName, parentRelation);
                }
                // Add relation
                relations.set(sourceClassName, relation);
                continue;
            }
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (keys_2_1 && !keys_2_1.done && (_a = keys_2.return)) _a.call(keys_2);
        }
        finally { if (e_2) throw e_2.error; }
    }
    return relations;
};
/**
 * Get the constraint key format of the supplied key
 * @param {String} key
 */
var getConstraintFrom = function (className, key) {
    // Check if the key is for a relation
    if (relation_1.default.isUsedBy(key))
        return key;
    else
        return relation_1.default.formatKey(className, key);
};
/**
 * Determine constraints
 * @param prefix
 */
exports.getConstraintsFrom = function (className, constraints) {
    var e_3, _a;
    // Create a new instance of where
    var where = new constraint_map_1.default(constraints.toJSON());
    // Remove deleted rows
    where.set(constants_1.DeletedAt, constraint_map_1.Constraints.Exists, false);
    try {
        // Iterate through keys
        for (var _b = __values(where.keys), _c = _b.next(); !_c.done; _c = _b.next()) {
            var key = _c.value;
            // Check if key is compound
            if (compound_key_1.default.isUsedBy(key)) {
                var keys = compound_key_1.default.from(key).map(function (k) { return getConstraintFrom(className, k); });
                where.changeKey(key, compound_key_1.default.toString(keys));
            }
            else
                where.changeKey(key, getConstraintFrom(className, key));
        }
    }
    catch (e_3_1) { e_3 = { error: e_3_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_3) throw e_3.error; }
    }
    return where;
};
/**
 * Get sorting
 */
function getSortingFrom(className, sort) {
    var e_4, _a;
    // Prepare sorting
    var sorting = [];
    try {
        // Iterate through each sort
        for (var sort_1 = __values(sort), sort_1_1 = sort_1.next(); !sort_1_1.done; sort_1_1 = sort_1.next()) {
            var sortKey = sort_1_1.value;
            // Order
            var order = sortKey[0] === constants_1.SortSymbol ? constants_1.SortDescending : constants_1.SortAscending;
            var key = order === constants_1.SortDescending ? sortKey.slice(1) : sortKey;
            // Add className to sort key if it is not a relation
            if (!relation_1.default.isUsedBy(key))
                key = "" + (order === constants_1.SortDescending ? constants_1.SortSymbol : '') + relation_1.default.formatKey(className, key);
            // Push the key
            sorting.push(key);
        }
    }
    catch (e_4_1) { e_4 = { error: e_4_1 }; }
    finally {
        try {
            if (sort_1_1 && !sort_1_1.done && (_a = sort_1.return)) _a.call(sort_1);
        }
        finally { if (e_4) throw e_4.error; }
    }
    return sorting;
}
exports.getSortingFrom = getSortingFrom;
//# sourceMappingURL=query-mapper.js.map