"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var enforce_js_1 = __importDefault(require("enforce-js"));
var class_1 = require("./class");
var key_map_1 = __importDefault(require("../../utils/key-map"));
var error_1 = __importDefault(require("../../utils/error"));
var constants_1 = require("../../utils/constants");
var constraint_map_1 = __importStar(require("../../utils/constraint-map"));
var format_1 = require("../../utils/format");
var query_mapper_1 = require("./query-mapper");
var Query = /** @class */ (function () {
    function Query(classType) {
        this.selection = [];
        this.included = [];
        this.constraints = new constraint_map_1.default;
        this.sorting = constants_1.Defaults.Query.Sort;
        this.skipped = constants_1.Defaults.Query.Skip;
        this.limitation = constants_1.Defaults.Query.Limit;
        this.classType = classType;
    }
    /**
     * Set a key constraint
     * @param {String} key
     * @param {String} constraint
     * @param {*} value
     */
    Query.prototype.set = function (key, constraint, value) {
        // Enforce
        enforce_js_1.default(templateObject_1 || (templateObject_1 = __makeTemplateObject(["", " as a string"], ["", " as a string"])), { key: key });
        // Check if the key exists for the class
        if (!this.class.has(key)) {
            var suggestion = this.class.has(format_1.toSnakeCase(key)) ? ", did you mean `" + format_1.toSnakeCase(key) + "`?" : '';
            throw new error_1.default(error_1.default.Code.ForbiddenOperation, "Constraint key `" + key + "` does not exist in `" + this.class.className + "`" + suggestion);
        }
        // Convert to string if value is a date
        if (value instanceof Date)
            value = format_1.toDatabaseDate(value.toISOString());
        // Set the constraint
        this.constraints.set(key, constraint, value);
        return this;
    };
    Object.defineProperty(Query.prototype, "class", {
        get: function () {
            return this.classType;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Assert that the key is an exact match to the given value
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.equalTo = function (key, value) {
        if (typeof value === 'boolean')
            value = value ? 1 : 0;
        this.set(key, constraint_map_1.Constraints.EqualTo, value);
        return this;
    };
    /**
     * Assert that the key is not an exact match to the given value
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.notEqualTo = function (key, value) {
        if (typeof value === 'boolean')
            value = value ? 1 : 0;
        this.set(key, constraint_map_1.Constraints.NotEqualTo, value);
        return this;
    };
    /**
     * Assert that the key is greater than the given value
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.greaterThan = function (key, value) {
        this.set(key, constraint_map_1.Constraints.GreaterThan, value);
        return this;
    };
    /**
     * Assert that the key is greater than or equal to the given value
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.greaterThanOrEqualTo = function (key, value) {
        this.set(key, constraint_map_1.Constraints.GreaterThanOrEqualTo, value);
        return this;
    };
    /**
     * Assert that the key is less than the given value
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.lessThan = function (key, value) {
        this.set(key, constraint_map_1.Constraints.LessThan, value);
        return this;
    };
    /**
     * Assert that the key is less than or equal to the given value
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.lessThanOrEqualTo = function (key, value) {
        this.set(key, constraint_map_1.Constraints.LessThanOrEqualTo, value);
        return this;
    };
    /**
     * Assert that the key is not null
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.exists = function (key) {
        this.set(key, constraint_map_1.Constraints.Exists, true);
        return this;
    };
    /**
     * Assert that the key is null
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.doesNotExist = function (key) {
        this.set(key, constraint_map_1.Constraints.Exists, false);
        return this;
    };
    /**
     * Assert that the key is one of the given values
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.containedIn = function (key, value) {
        this.set(key, constraint_map_1.Constraints.ContainedIn, value);
        return this;
    };
    /**
     * Assert that the key is not any of the given values
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.notContainedIn = function (key, value) {
        this.set(key, constraint_map_1.Constraints.NotContainedIn, value);
        return this;
    };
    /**
     * Assert that the key is either one of the values or is null
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.containedInOrDoesNotExist = function (key, value) {
        this.set(key, constraint_map_1.Constraints.ContainedInOrDoesNotExist, value);
        return this;
    };
    /**
     * Assert that the key starts with the given string
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.startsWith = function (key, value) {
        this.set(key, constraint_map_1.Constraints.StartsWith, value);
        return this;
    };
    /**
     * Assert that the key ends with the given string
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.endsWith = function (key, value) {
        this.set(key, constraint_map_1.Constraints.EndsWith, value);
        return this;
    };
    /**
     * Assert that the key contains the given string
     * @param {String} key
     * @param {String} value
     */
    Query.prototype.contains = function (key, value) {
        if (key instanceof Array)
            key = key.join('|');
        this.set(key, constraint_map_1.Constraints.Contains, value);
        return this;
    };
    /**
     * Assert that the key contains either of the given strings
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.containsEither = function (key, value) {
        if (key instanceof Array)
            key = key.join('|');
        this.set(key, constraint_map_1.Constraints.ContainsEither, value);
        return this;
    };
    /**
     * Assert that the key contains all of the given strings
     * @param {String} key
     * @param {*} value
     */
    Query.prototype.containsAll = function (key, value) {
        if (key instanceof Array)
            key = key.join('|');
        this.set(key, constraint_map_1.Constraints.ContainsAll, value);
        return this;
    };
    /**
     * Assert that the key matches a key in a subquery
     * @param {String} key
     * @param {String} select
     * @param {Object} value
     */
    Query.prototype.foundIn = function (key, select, value) {
        // Set constraint
        this.set(key, constraint_map_1.Constraints.FoundIn, value.toSubquery(select));
        return this;
    };
    /**
     * Assert that the key matches a key in any of the given subqueries
     * @param {String} key
     * @param {Array} value
     */
    Query.prototype.foundInEither = function (key, value) {
        this.set(key, constraint_map_1.Constraints.FoundInEither, value.map(function (item) {
            var select = Object.keys(item)[0];
            var query = item[select];
            return query.toSubquery(select);
        }));
        return this;
    };
    /**
     * Assert that the key matches a key in all of the given subqueries
     * @param {String} key
     * @param {Array} value
     */
    Query.prototype.foundInAll = function (key, value) {
        this.set(key, constraint_map_1.Constraints.FoundInAll, value.map(function (item) {
            var select = Object.keys(item)[0];
            var query = item[select];
            return query.toSubquery(select);
        }));
        return this;
    };
    /**
     * Assert that the key does not match a key in the given subquery
     * @param {String} key
     * @param {String} select
     * @param {Object} value
     */
    Query.prototype.notFoundIn = function (key, select, value) {
        // Set constraint
        this.set(key, constraint_map_1.Constraints.NotFoundIn, value.toSubquery(select));
        return this;
    };
    /**
     * Assert that the key does not match a key in either of the given subqueries
     * @param {String} key
     * @param {Array} value
     */
    Query.prototype.notFoundInEither = function (key, value) {
        this.set(key, constraint_map_1.Constraints.NotFoundInEither, value.map(function (item) {
            var select = Object.keys(item)[0];
            var query = item[select];
            return query.toSubquery(select);
        }));
        return this;
    };
    /**
     * Assert that the key does not match a key in all of the given subqueries
     * @param {String} key
     * @param {Array} value
     */
    Query.prototype.notFoundInAll = function (key, value) {
        this.set(key, constraint_map_1.Constraints.NotFoundInAll, value.map(function (item) {
            var select = Object.keys(item)[0];
            var query = item[select];
            return query.toSubquery(select);
        }));
        return this;
    };
    Query.prototype.select = function () {
        var keys = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            keys[_i] = arguments[_i];
        }
        var e_1, _a;
        // Check if first key is an array
        if (keys.length === 0)
            throw new error_1.default(error_1.default.Code.MissingConfiguration, 'Select key must be a string or an array of strings');
        var keyList = keys[0] instanceof Array ? keys[0] : keys;
        try {
            // Loop through the keys
            for (var keyList_1 = __values(keyList), keyList_1_1 = keyList_1.next(); !keyList_1_1.done; keyList_1_1 = keyList_1.next()) {
                var key = keyList_1_1.value;
                enforce_js_1.default(templateObject_2 || (templateObject_2 = __makeTemplateObject(["", " as a string"], ["", " as a string"])), { key: key });
                // Check if the key exists for the class
                if (!this.class.has(key)) {
                    var suggestion = this.class.has(format_1.toSnakeCase(key)) ? ", did you mean `" + format_1.toSnakeCase(key) + "`?" : '';
                    throw new error_1.default(error_1.default.Code.InvalidObjectKey, "Select key `" + key + "` does not exist in `" + this.class.className + "`" + suggestion);
                }
                this.selection.push(key);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (keyList_1_1 && !keyList_1_1.done && (_a = keyList_1.return)) _a.call(keyList_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return this;
    };
    Query.prototype.include = function () {
        var keys = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            keys[_i] = arguments[_i];
        }
        var e_2, _a;
        // Check if first key is an array
        if (!keys)
            throw new error_1.default(error_1.default.Code.MissingConfiguration, 'Include key must be a string or an array of strings');
        var keyList = keys[0] instanceof Array ? keys[0] : keys;
        try {
            // Loop through the keys
            for (var keyList_2 = __values(keyList), keyList_2_1 = keyList_2.next(); !keyList_2_1.done; keyList_2_1 = keyList_2.next()) {
                var key = keyList_2_1.value;
                enforce_js_1.default(templateObject_3 || (templateObject_3 = __makeTemplateObject(["", " as a string"], ["", " as a string"])), { key: key });
                // Check if the key exists for the class
                if (!this.class.has(key)) {
                    var suggestion = this.class.has(format_1.toSnakeCase(key)) ? ", did you mean `" + format_1.toSnakeCase(key) + "`?" : '';
                    throw new error_1.default(error_1.default.Code.InvalidObjectKey, "Include key `" + key + "` does not exist in `" + this.class.className + "`" + suggestion);
                }
                this.included.push(key);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (keyList_2_1 && !keyList_2_1.done && (_a = keyList_2.return)) _a.call(keyList_2);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return this;
    };
    /**
     * Sort the query by the provided keys in ascending order
     * @param {String} keys
     */
    Query.prototype.sortBy = function () {
        var keys = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            keys[_i] = arguments[_i];
        }
        var e_3, _a;
        // Check if first key is an array
        if (!keys)
            throw new error_1.default(error_1.default.Code.MissingConfiguration, 'SortBy key must be a string or an array of strings');
        var keyList = keys[0] instanceof Array ? keys[0] : keys;
        try {
            // Loop through the keys
            for (var keyList_3 = __values(keyList), keyList_3_1 = keyList_3.next(); !keyList_3_1.done; keyList_3_1 = keyList_3.next()) {
                var key = keyList_3_1.value;
                enforce_js_1.default(templateObject_4 || (templateObject_4 = __makeTemplateObject(["", " as a string"], ["", " as a string"])), { key: key });
                // Get rawKey
                var rawKey = key[0] === '-' ? key.substr(1) : key;
                // Check if the key exists for the class
                if (!this.class.has(rawKey)) {
                    var suggestion = this.class.has(format_1.toSnakeCase(key)) ? ", did you mean `" + format_1.toSnakeCase(key) + "`?" : '';
                    throw new error_1.default(error_1.default.Code.InvalidObjectKey, "Sort key `" + key + "` does not exist in `" + this.class.className + "`" + suggestion);
                }
                this.sorting.push(key);
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (keyList_3_1 && !keyList_3_1.done && (_a = keyList_3.return)) _a.call(keyList_3);
            }
            finally { if (e_3) throw e_3.error; }
        }
        return this;
    };
    /**
     * Sort the query by the provided keys in descending order
     * @param {String} keys
     */
    Query.prototype.sortByDescending = function () {
        var keys = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            keys[_i] = arguments[_i];
        }
        var e_4, _a;
        // Check if first key is an array
        if (!keys)
            throw new error_1.default(error_1.default.Code.MissingConfiguration, 'SortByDescending key must be a string or an array of strings');
        var keyList = keys[0] instanceof Array ? keys[0] : keys;
        try {
            // Loop through the keys
            for (var keyList_4 = __values(keyList), keyList_4_1 = keyList_4.next(); !keyList_4_1.done; keyList_4_1 = keyList_4.next()) {
                var key = keyList_4_1.value;
                enforce_js_1.default(templateObject_5 || (templateObject_5 = __makeTemplateObject(["", " as a string"], ["", " as a string"])), { key: key });
                // Check if the key exists for the class
                if (!this.class.has(key)) {
                    var suggestion = this.class.has(format_1.toSnakeCase(key)) ? ", did you mean `" + format_1.toSnakeCase(key) + "`?" : '';
                    throw new error_1.default(error_1.default.Code.InvalidObjectKey, "Sort key `" + key + "` does not exist in `" + this.class.className + "`" + suggestion);
                }
                this.sorting.push("-" + key);
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (keyList_4_1 && !keyList_4_1.done && (_a = keyList_4.return)) _a.call(keyList_4);
            }
            finally { if (e_4) throw e_4.error; }
        }
        return this;
    };
    /**
     * Number of items to skip for the query
     * @param {String} keys
     */
    Query.prototype.skip = function (value) {
        enforce_js_1.default(templateObject_6 || (templateObject_6 = __makeTemplateObject(["", " as a number, greater than or equal to 0"], ["", " as a number, greater than or equal to 0"])), { skip: value });
        this.skipped = value;
        return this;
    };
    /**
     * Number of items to fetch, at maximum
     * @param {String} keys
     */
    Query.prototype.limit = function (value) {
        enforce_js_1.default(templateObject_7 || (templateObject_7 = __makeTemplateObject(["", " as a number, greater than or equal to 0"], ["", " as a number, greater than or equal to 0"])), { limit: value });
        this.limitation = value;
        return this;
    };
    /**
     * Convert the query into a subquery
     * @param {String} select
     */
    Query.prototype.toSubquery = function (select) {
        this.selection = [];
        this.select(select);
        return this;
    };
    /**
     * Generic where clause
     * @param constraints
     */
    Query.prototype.where = function (constraints) {
        var e_5, _a, e_6, _b;
        try {
            // Iterate through constraints
            for (var _c = __values(Object.entries(constraints)), _d = _c.next(); !_d.done; _d = _c.next()) {
                var _e = __read(_d.value, 2), key = _e[0], constraintMap = _e[1];
                try {
                    for (var _f = __values(Object.entries(constraintMap)), _g = _f.next(); !_g.done; _g = _f.next()) {
                        var _h = __read(_g.value, 2), constraint = _h[0], value = _h[1];
                        this.set(key, constraint, value);
                    }
                }
                catch (e_6_1) { e_6 = { error: e_6_1 }; }
                finally {
                    try {
                        if (_g && !_g.done && (_b = _f.return)) _b.call(_f);
                    }
                    finally { if (e_6) throw e_6.error; }
                }
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_5) throw e_5.error; }
        }
    };
    /**
     * Get selection
     */
    Query.prototype.getSelection = function () {
        // Get definition
        var definition = class_1.ClassDefinitionManager.get(this.class);
        // Get selection
        var defaultSelect = __spread([constants_1.InternalId], definition.keys, definition.timestamps);
        var selected = this.selection.length > 0 ? this.selection : defaultSelect;
        var selection = __spread(selected, this.included);
        return selection;
    };
    /**
     * Create class from keyMap
     * @param keys
     */
    Query.prototype.getClassFromKeys = function (keys) {
        var e_7, _a;
        // Get definition
        var definition = class_1.ClassDefinitionManager.get(this.class);
        // Prepare new key map
        var keyMap = new key_map_1.default;
        // Get internal keys
        var id = keys.get(constants_1.InternalId);
        // Remove id from the key map
        keys.remove(constants_1.InternalId);
        try {
            // Iterate through each key
            for (var _b = __values(keys.toArray()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = __read(_c.value, 2), key = _d[0], value = _d[1];
                var relationDefinition = definition.relations[key];
                if (typeof relationDefinition !== 'undefined') {
                    var relation = relationDefinition.toRelation();
                    keyMap.set(key, new relation.class(value));
                }
                else
                    keyMap.set(key, value);
            }
        }
        catch (e_7_1) { e_7 = { error: e_7_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_7) throw e_7.error; }
        }
        // Return the new class
        var classInstance = (new this.class);
        classInstance.identifier = id;
        classInstance.keys = keyMap;
        return classInstance;
    };
    /**
     * Convert query into options for database
     */
    Query.prototype.toQueryOptions = function () {
        // Get class alias
        var className = this.class.className;
        // Get class details
        var relationsMap = class_1.ClassDefinitionManager.get(this.class).relations;
        // Get selection
        var selection = this.getSelection();
        // Get columns
        var columns = query_mapper_1.getColumnsFrom(className, selection, relationsMap);
        // Get relations
        var relations = query_mapper_1.getRelationsFrom(selection, relationsMap);
        // Get where constraints
        var constraints = query_mapper_1.getConstraintsFrom(className, this.constraints);
        // Get sorting
        var sorting = query_mapper_1.getSortingFrom(className, this.sorting);
        // Get pagination
        var _a = this, skipped = _a.skipped, limitation = _a.limitation;
        // Return query options
        return {
            source: [this.class.source, className],
            columns: columns,
            relations: relations,
            constraints: constraints,
            sorting: sorting,
            skipped: skipped,
            limitation: limitation,
        };
    };
    return Query;
}());
exports.default = Query;
var templateObject_1, templateObject_2, templateObject_3, templateObject_4, templateObject_5, templateObject_6, templateObject_7;
//# sourceMappingURL=query.js.map