"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var class_1 = require("./class");
var constants_1 = require("../../utils/constants");
var error_1 = __importDefault(require("../../utils/error"));
var Relation = /** @class */ (function () {
    /**
     * Constructor
     * @param {Class} classType
     * @param {String} aliasKey
     */
    function Relation(classType, sourceClassName, sourceKey, parentClassName, parentKey, secondary) {
        if (secondary === void 0) { secondary = false; }
        this.classType = classType;
        this.sourceClassName = sourceClassName;
        this.sourceKey = sourceKey;
        this.parentClassName = parentClassName;
        this.parentKey = parentKey;
        this.secondaryFlag = secondary;
    }
    Relation.isUsedBy = function (key) {
        return key.indexOf(constants_1.RelationDelimiter) > 0;
    };
    Relation.isValid = function (key) {
        return this.isUsedBy(key) && key.split(constants_1.RelationDelimiter).length === 2;
    };
    Relation.parseKey = function (keyName) {
        // Get key parts
        var _a = __read(keyName.split(constants_1.RelationDelimiter, 2), 2), className = _a[0], key = _a[1];
        return [className, key];
    };
    Relation.formatKey = function (className, key) {
        return "" + className + constants_1.RelationDelimiter + key;
    };
    Relation.formatAsId = function (key) {
        return key + "_" + constants_1.InternalId;
    };
    Relation.prototype.statics = function () {
        return this.constructor;
    };
    Object.defineProperty(Relation.prototype, "class", {
        get: function () {
            return this.classType;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Relation.prototype, "secondary", {
        get: function () {
            return this.secondaryFlag;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * The foreign key inside the current class
     */
    Relation.prototype.sourceClassKey = function (className) {
        var sourceClass = this.sourceClassName !== RelationDefinition.OwnerSymbol ? this.sourceClassName : className;
        return this.statics().formatKey(sourceClass, this.sourceKey);
    };
    /**
     * The key inside the other class that the foreign key matches
     */
    Relation.prototype.parentClassKey = function () {
        return this.statics().formatKey(this.parentClassName, this.parentKey);
    };
    Relation.prototype.isImplementedBy = function (value) {
        if (value === null)
            return true;
        if (typeof value !== 'object')
            return false;
        if (value[constants_1.InternalKeys.Relations.Type] !== constants_1.RelationTypeName)
            return false;
        if (value[constants_1.InternalKeys.Relations.ClassName] !== this.class.className)
            return false;
        if (value[constants_1.InternalId] === null || typeof value[constants_1.InternalId] === 'undefined')
            return false;
        return true;
    };
    return Relation;
}());
exports.default = Relation;
var RelationDefinition = /** @class */ (function () {
    function RelationDefinition(classDefinition, sourceClassName, sourceKey, parentClassName, parentKey) {
        this.classCaller = classDefinition;
        this.sourceClassName = sourceClassName;
        this.sourceKey = sourceKey;
        this.parentClassName = parentClassName;
        this.parentKey = parentKey;
    }
    RelationDefinition.prototype.toRelation = function () {
        var classType = this.classCaller();
        var definition = class_1.ClassDefinitionManager.get(classType);
        var _a = this, sourceClassName = _a.sourceClassName, sourceKey = _a.sourceKey, parentClassName = _a.parentClassName, parentKey = _a.parentKey;
        var actualSourceKey = sourceKey;
        var secondary = false;
        // Check if source is the owner
        if (sourceClassName !== RelationDefinition.OwnerSymbol)
            secondary = true;
        // Check if relation is secondary
        if (secondary) {
            if (typeof definition.relations[sourceKey] !== 'undefined') {
                // Get parent definition
                var parentDefinition = definition.relations[sourceKey];
                actualSourceKey = parentDefinition.toRelation().sourceKey;
            }
            else
                throw new error_1.default(error_1.default.Code.ForbiddenOperation, "Relation `" + this.sourceKey + "` does not exist in " + sourceClassName);
        }
        // Create a relation
        var relation = new Relation(classType, sourceClassName, actualSourceKey, parentClassName, parentKey, secondary);
        // Return relation
        return relation;
    };
    RelationDefinition.OwnerSymbol = '*';
    return RelationDefinition;
}());
exports.RelationDefinition = RelationDefinition;
//# sourceMappingURL=relation.js.map