"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var express_1 = __importDefault(require("express"));
var cors_1 = __importDefault(require("cors"));
var body_parser_1 = __importDefault(require("body-parser"));
var error_1 = __importDefault(require("../utils/error"));
var constants_1 = require("../utils/constants");
var middleware = function (api) {
    /**
     * Define router
     */
    var router = express_1.default.Router();
    /**
     * Enable CORS
     */
    router.use(cors_1.default());
    /**
     * Body parser
     */
    router.use(body_parser_1.default.json());
    router.use(body_parser_1.default.urlencoded({ extended: false }));
    /**
     * Require API Keys for all requests
     */
    router.use(function (req, res, next) {
        var key = req.get(constants_1.InternalKeys.Middleware.ApiKey);
        if (!key || key !== api.apiKey) {
            var error = new error_1.default(error_1.default.Code.InvalidAPIKey, 'Invalid API Key');
            next(error);
        }
        else
            next();
    });
    /**
     * Add the data mapper to req.classes
     */
    router.use(function (req, res, next) {
        // Add the data mapper to the request
        req[constants_1.InternalKeys.Middleware.ClassManager] = api.classes;
        next();
    });
    // Return the router
    return router;
};
exports.default = middleware;
//# sourceMappingURL=middleware.js.map