import Class from '../features/orm/class';
export default class Collection<T extends Class> {
    private collection;
    constructor(collection: T[]);
    readonly length: number;
    count(): number;
    /**
     * Get the first item from the collection
     */
    first(): T | null;
    /**
     * Get the last Object from the collection
     */
    last(): T | null;
    /**
     * Return Objects that pass a given evaluator
     * @param {Function} evaluator
     */
    where(evaluator: (object: T) => boolean): Collection<T>;
    /**
     * Map Objects into an array using an iterator
     * @param {Function} iterator
     */
    map<U extends any>(iterator: (object: T) => any): U[];
    /**
     * Iterate through each item
     * @param {Function} iterator
     */
    forEach(iterator: (object: T) => void): void;
    /**
     * Alias of toArray()
     */
    toList(): T[];
    /**
     * Convert the collection into an array
     */
    toArray(): T[];
    /**
     * Convert the collection into a Map, based on the given iterator
     * @param iterator
     */
    toMap(iterator?: string | ((object: T) => any)): Map<any, T>;
    /**
     * Convert the collection into an object literal
     */
    toJSON(): import("../types/class").ClassJSON[];
    /**
     * Run a promise iterator over every Object, in series
     * @param {Function} iterator
     */
    each(iterator: (object: T) => Promise<any>): Promise<void>;
    /**
     * Run a promise iterator over every Object, in parallel
     */
    all(iterator: (object: T) => Promise<any>): Promise<void>;
    [Symbol.iterator](): Iterator<T>;
}
