"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var error_1 = __importDefault(require("./error"));
var KeyConstraints = /** @class */ (function () {
    /**
     * Constructor
     * @param {String} key
     * @param {Object} list
     * @param {String} key
     */
    function KeyConstraints(key, map) {
        // Set values
        this.keyName = key;
        this.map = map;
    }
    Object.defineProperty(KeyConstraints.prototype, "key", {
        get: function () {
            return this.keyName;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyConstraints.prototype, "constraints", {
        get: function () {
            var _this = this;
            return Object.keys(this.map).map(function (constraint) {
                return { key: _this.key, constraint: constraint, value: _this.map[constraint] };
            });
        },
        enumerable: true,
        configurable: true
    });
    KeyConstraints.prototype.set = function (constraint, value) {
        this.map[constraint] = value;
    };
    KeyConstraints.prototype.changeKey = function (newKey) {
        this.keyName = newKey;
    };
    KeyConstraints.prototype.toJSON = function () {
        return __assign({}, this.map);
    };
    return KeyConstraints;
}());
exports.KeyConstraints = KeyConstraints;
exports.Subqueries = Object.freeze({
    FoundIn: 'fi',
    FoundInEither: 'fie',
    FoundInAll: 'fia',
    NotFoundIn: 'nfi',
    NotFoundInEither: 'nfe',
    NotFoundInAll: 'nfa',
});
exports.Constraints = Object.freeze(__assign({ EqualTo: 'eq', NotEqualTo: 'neq', GreaterThan: 'gt', GreaterThanOrEqualTo: 'gte', LessThan: 'lt', LessThanOrEqualTo: 'lte', Exists: 'ex', ContainedIn: 'in', NotContainedIn: 'nin', ContainedInOrDoesNotExist: 'inx', StartsWith: 'str', EndsWith: 'end', Contains: 'has', ContainsEither: 'hasi', ContainsAll: 'hasa' }, exports.Subqueries));
var ConstraintMap = /** @class */ (function () {
    /**
     * Constructor
     * @param {Object} keyValuePairs
     */
    function ConstraintMap(constraints) {
        if (constraints === void 0) { constraints = {}; }
        /**
         * Private properties
         */
        this.map = {};
        // Populate map
        for (var key in constraints) {
            this.map[key] = new KeyConstraints(key, constraints[key]);
        }
    }
    Object.defineProperty(ConstraintMap.prototype, "keys", {
        get: function () {
            return Object.keys(this.map);
        },
        enumerable: true,
        configurable: true
    });
    ConstraintMap.prototype.set = function (key, constraint, value) {
        var constraints = this.map[key] || new KeyConstraints(key, {});
        constraints.set(constraint, value);
        this.map[key] = constraints;
    };
    ConstraintMap.prototype.changeKey = function (key, newKey) {
        var constraints = this.map[key];
        if (!constraints)
            throw new error_1.default(error_1.default.Code.MissingConfiguration, "Constraint key being changed does not exist: `" + key + "`");
        // Don't change keys if they are the same
        if (key === newKey)
            return;
        constraints.changeKey(newKey);
        this.map[newKey] = constraints;
        delete this.map[key];
    };
    ConstraintMap.prototype.get = function (key) {
        return this.map[key];
    };
    ConstraintMap.prototype.toArray = function () {
        var _this = this;
        return Object.keys(this.map).map(function (key) { return _this.map[key]; });
    };
    ConstraintMap.prototype.toJSON = function () {
        var _this = this;
        return Object.keys(this.map).reduce(function (map, key) {
            map[key] = _this.map[key].toJSON();
            return map;
        }, {});
    };
    return ConstraintMap;
}());
exports.default = ConstraintMap;
//# sourceMappingURL=constraint-map.js.map