"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var error_1 = __importDefault(require("./error"));
var KeyMap = /** @class */ (function () {
    /**
     * Constructor
     * @param {Object} keyValues
     * @param {Boolean} immutable
     */
    function KeyMap(keyValues, immutable) {
        if (keyValues === void 0) { keyValues = {}; }
        if (immutable === void 0) { immutable = false; }
        /**
         * Private properties
         */
        this.map = new Map([]);
        // Populate map
        this.map = new Map(Object.entries(keyValues));
        // Set immutability
        this.immutable = immutable;
    }
    Object.defineProperty(KeyMap.prototype, "length", {
        get: function () {
            return Object.keys(this.map).length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyMap.prototype, "keys", {
        get: function () {
            return __spread(this.map.keys());
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyMap.prototype, "values", {
        get: function () {
            return __spread(this.map.values());
        },
        enumerable: true,
        configurable: true
    });
    KeyMap.prototype.has = function (key) {
        return this.map.has(key);
    };
    KeyMap.prototype.set = function (key, value) {
        // Check if immutable
        if (this.immutable)
            throw new error_1.default(error_1.default.Code.ForbiddenOperation, 'Cannot set a value for an immutable KeyMap');
        // Set the key value
        this.map.set(key, value);
    };
    KeyMap.prototype.get = function (key) {
        // Return the value
        return this.map.get(key);
    };
    KeyMap.prototype.remove = function (key) {
        // Check if immutable
        if (this.immutable)
            throw new error_1.default(error_1.default.Code.ForbiddenOperation, 'Cannot remove from an immutable KeyMap');
        // Remove the key from the map
        this.map.delete(key);
    };
    KeyMap.prototype.toArray = function () {
        return __spread(this.map.entries());
    };
    KeyMap.prototype.freeze = function () {
        this.immutable = true;
    };
    KeyMap.prototype.toJSON = function () {
        // Get json
        var json = this.toArray().reduce(function (map, _a) {
            var _b = __read(_a, 2), key = _b[0], value = _b[1];
            var _c;
            return (__assign({}, map, (_c = {}, _c[key] = value, _c)));
        }, {});
        // Set immutability
        if (this.immutable)
            return Object.freeze(json);
        // Return json
        return json;
    };
    return KeyMap;
}());
exports.default = KeyMap;
//# sourceMappingURL=key-map.js.map